﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using ABB.Robotics.Controllers;
using ABB.Robotics.Controllers.Discovery;

namespace GuidControllerScan
{
    class Program
    {
        private static readonly int WAIT_TIME = 1000;
        private static readonly int RETRIES = 2;
        private static readonly string PATH_GUID = "C:\\TestGuid\\guid.txt";

        static void Main(string[] args)
        {
            //SaveControllers();
            LoadControllers();
            Console.ReadKey();
        }

        // Save the scanned controllers to the Guid file
        public static void SaveControllers()
        {
            ControllerInfo[] cInf = ScanControllerInfo();
            Guid[] guid = cInf.Select(e => new Guid(e.SystemId.ToString())).ToArray();
            WriteGuidLog(guid, PATH_GUID);
        }

        // Load the controllers from the saved Guid file
        public static void LoadControllers()
        {
            Guid[] guid = ReadGuidLog(PATH_GUID);
            ControllerInfo[] cInf = GetControllers(guid);
        }

        /// Attempts to get the controllers from an array of Guid
        public static ControllerInfo[] GetControllers(Guid[] guidArray)
        {
            Console.WriteLine($"Scanning for {guidArray.Length} Guid...");
            NetworkScanner networkScanner = new NetworkScanner();

            List<ControllerInfo> controllerInfoList = new List<ControllerInfo>();

            for (int i = 0; i < guidArray.Length; i++)
                if (networkScanner.TryFind(guidArray[i], WAIT_TIME, RETRIES, out ControllerInfo info))
                    controllerInfoList.Add(info);

            Console.WriteLine($"Found {controllerInfoList.Count} ControllerInfo.");

            return controllerInfoList.ToArray();
        }

        /// Save the Guid as a text file
        private static void WriteGuidLog(Guid[] guidArray, string filePath)
        {
            Console.WriteLine($"Writing Guid log {filePath}");
            FileInfo info = new FileInfo(filePath);
            if (info.Exists)
                info.Delete();
            else
                info.Directory.Create();
            File.WriteAllLines(filePath, guidArray.Select(e => e.ToString()).ToArray());
            Console.WriteLine($"Finished writing Guid log {filePath}");
        }

        /// Read Guid from a text file
        private static Guid[] ReadGuidLog(string filePath)
        {
            Console.WriteLine($"Reading Guid log {filePath}");
            return File.ReadAllLines(filePath).Select(e => new Guid(e)).ToArray();
        }

        /// Scan the network for controllers
        private static ControllerInfo[] ScanControllerInfo()
        {
            Console.WriteLine($"Scanning for controllers...");
            NetworkScanner scanner = new NetworkScanner();
            scanner.Scan();
            // Wait a while for the scan to find more controllers if needed.
            System.Threading.Thread.Sleep(4000);
            scanner.Scan();
            Console.WriteLine($"Found {scanner.GetControllers().Length} controllers.");
            return scanner.GetControllers();
        }
    }
}